"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Subject_1 = require("rxjs/Subject");
const ReplaySubject_1 = require("rxjs/ReplaySubject");
require("rxjs/add/operator/map");
require("rxjs/add/operator/filter");
require("rxjs/add/operator/distinct");
var StoRx;
(function (StoRx) {
    function createStore(state) {
        return new StoreImpl(state);
    }
    StoRx.createStore = createStore;
    class StoreUtils {
        static createMapFunction(path) {
            let pathSplit = path.split(".");
            return s => {
                let current = s;
                for (let el of pathSplit) {
                    current = current[el];
                }
                return current;
            };
        }
        static createMapReverseFunction(path) {
            let pathSplit = path.split(".");
            return (s, p) => {
                let current = p;
                for (let i = 0; i < pathSplit.length - 1; i++) {
                    current = current[pathSplit[i]];
                }
                current[pathSplit[pathSplit.length - 1]] = s;
                return p;
            };
        }
    }
    class StoreImpl {
        constructor(state) {
            this.state = state;
            this.stateSubject = new ReplaySubject_1.ReplaySubject(1);
            this.actionSubject = new Subject_1.Subject();
            this.updateState(state);
        }
        updateState(state) {
            this.currentState = state;
            this.stateSubject.next(state);
        }
        subscribeReducer(actions, reducer) {
            actions.subscribe(a => {
                let state = this.currentState;
                let newState = reducer.reduce(state, a);
                if (state != newState) {
                    this.updateState(newState);
                }
            });
        }
        subscribe(actions, reducer) {
            this.subscribeReducer(actions, new ReducerFunction(reducer));
        }
        subscribeEvent(actions, reducer) {
            this.subscribe(actions.map(e => e.event), reducer);
        }
        subscribeReducerEvent(actions, reducer) {
            this.subscribeReducer(actions.map(e => e.event), reducer);
        }
        mapFunction(id, map, mapReverse) {
            return new ViewStore(id, map, mapReverse, this);
        }
        map(path) {
            return new ViewStore(path, StoreUtils.createMapFunction(path), StoreUtils.createMapReverseFunction(path), this);
        }
        observable() {
            return this.stateSubject;
        }
        dispatch(action) {
            this.dispatchWithSource(action, '');
        }
        dispatchWithSource(action, source) {
            this.actionSubject.next({ source: source, event: action });
        }
        action() {
            return new ActionManagerImpl(this, this.actionSubject);
        }
        actionByType(type) {
            return new ActionManagerImpl(this, this.actionSubject.asObservable().filter(v => v.event instanceof type));
        }
        actions() {
            return this.actionSubject;
        }
    }
    class ActionManagerImpl {
        constructor(store, obs) {
            this.store = store;
            this.obs = obs;
        }
        subscribeReducer(reducer) {
            this.getStoreDispatcher().subscribeReducerEvent(this.observableEvent(), reducer);
        }
        subscribe(reducer) {
            this.getStoreDispatcher().subscribeEvent(this.observableEvent(), reducer);
        }
        observable() {
            return this.obs.map(e => e.event);
        }
        observableEvent() {
            return this.obs;
        }
        filter(filter) {
            return new FilterActionManager(this, filter);
        }
        getStore() {
            return this.store;
        }
        getStoreDispatcher() {
            return this.store;
        }
    }
    class FilterActionManager {
        constructor(parent, filterFunction) {
            this.parent = parent;
            this.filterFunction = filterFunction;
        }
        subscribeReducer(reducer) {
            this.getStoreDispatcher().subscribeReducerEvent(this.observableEvent(), reducer);
        }
        subscribe(reducer) {
            this.getStoreDispatcher().subscribeEvent(this.observableEvent(), reducer);
        }
        observable() {
            return this.parent.observable().filter(this.filterFunction);
        }
        observableEvent() {
            return this.parent.observableEvent().filter(e => this.filterFunction(e.event));
        }
        filter(filter) {
            return new FilterActionManager(this, filter);
        }
        getStore() {
            return this.parent.getStore();
        }
        getStoreDispatcher() {
            return this.parent.getStoreDispatcher();
        }
    }
    class ViewStore {
        constructor(id, mapFunc, mapReverseFunction, store) {
            this.id = id;
            this.mapFunc = mapFunc;
            this.mapReverseFunction = mapReverseFunction;
            this.store = store;
        }
        observable() {
            return this.store.observable().map(this.mapFunc).distinct();
        }
        dispatch(action) {
            this.store.dispatchWithSource(action, this.id);
        }
        dispatchWithSource(action, source) {
            this.store.dispatchWithSource(action, source);
        }
        action() {
            return new ActionManagerImpl(this, this.store.actions());
        }
        actionByType(type) {
            return new ActionManagerImpl(this, this.store.actions().filter(v => v.event instanceof type));
        }
        subscribe(actions, reducer) {
            this.store.subscribeReducer(actions, new ViewReducer(this.mapFunc, this.mapReverseFunction, new ReducerFunction(reducer)));
        }
        subscribeReducer(actions, reducer) {
            this.store.subscribeReducer(actions, new ViewReducer(this.mapFunc, this.mapReverseFunction, reducer));
        }
        subscribeEvent(actions, reducer) {
            this.store.subscribeReducerEvent(actions.filter(event => event.source.startsWith(this.id)), new ViewReducer(this.mapFunc, this.mapReverseFunction, new ReducerFunction(reducer)));
        }
        subscribeReducerEvent(actions, reducer) {
            this.store.subscribeReducerEvent(actions.filter(event => event.source.startsWith(this.id)), new ViewReducer(this.mapFunc, this.mapReverseFunction, reducer));
        }
        mapFunction(id, map, mapReverse) {
            return new ViewStore(this.id + "." + id, map, mapReverse, this);
        }
        map(path) {
            return new ViewStore(this.id + "." + path, StoreUtils.createMapFunction(path), StoreUtils.createMapReverseFunction(path), this);
        }
        actions() {
            return this.store.actions();
        }
    }
    class ViewReducer {
        constructor(map, mapReverse, reducer) {
            this.map = map;
            this.mapReverse = mapReverse;
            this.reducer = reducer;
        }
        reduce(parentState, a) {
            let state = this.map(parentState);
            let reduceState = this.reducer.reduce(state, a);
            let result = parentState;
            if (state != reduceState) {
                result = this.mapReverse(reduceState, Object.assign({}, parentState));
            }
            return result;
        }
    }
    class ReducerFunction {
        constructor(reducer) {
            this.reducer = reducer;
        }
        reduce(state, a) {
            return this.reducer(state, a);
        }
    }
})(StoRx = exports.StoRx || (exports.StoRx = {}));
//# sourceMappingURL=Store.js.map