"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const Store_1 = require("../../src/store/Store");
var createStore = Store_1.StoRx.createStore;
class EventName {
    constructor(name) {
        this.name = name;
    }
}
class StateNameReducer {
    reduce(state, a) {
        let n = Object.assign({}, state);
        n.name = a.name;
        n.count++;
        return n;
    }
}
class NameReducer {
    reduce(state, a) {
        return a.name;
    }
}
describe("Store test", () => {
    it('creates store', () => {
        chai_1.expect(createStore({})).to.be.any;
    });
    it('store change any action value using function', () => {
        let initState = { name: "1", count: 0 };
        let store = createStore(initState);
        store.action().subscribe((s, a) => {
            let ns = Object.assign({}, s);
            ns.name = a.name;
            ns.count++;
            return ns;
        });
        let state = null;
        store.observable().subscribe(s => state = s);
        chai_1.expect(state).to.be.eq(initState);
        store.dispatch({ name: "test" });
        chai_1.expect(state).to.not.eq(initState);
        chai_1.expect(state.name).to.be.eq("test");
        chai_1.expect(state.count).to.be.eq(1);
    });
    it('store change any action value using reducer', () => {
        let initState = { name: "1", count: 0 };
        let store = createStore(initState);
        store.actionByType(EventName).subscribeReducer(new StateNameReducer());
        let state = null;
        store.observable().subscribe(s => state = s);
        chai_1.expect(state).to.be.eq(initState);
        store.dispatch(new EventName("test"));
        chai_1.expect(state).to.not.eq(initState);
        chai_1.expect(state.name).to.be.eq("test");
        chai_1.expect(state.count).to.be.eq(1);
    });
    it('store change any action value subStore', () => {
        let initState = { name: "1", count: 0 };
        let store = createStore(initState);
        let substore = store.map('name');
        substore.action().subscribe((s, a) => { return a.name; });
        let state = null;
        let substate = null;
        store.observable().subscribe(s => state = s);
        substore.observable().subscribe(s => substate = s);
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch({ name: "test" });
        chai_1.expect(substate).to.be.eq('test', "name in substate should be change by dispatch action");
        chai_1.expect(state).to.be.not.eq(initState, "the state should be a new one s");
        chai_1.expect(state.name).to.be.eq("test", "name should be change by dispatch action");
    });
    it('subStore reducer does nothing', () => {
        let initState = { name: "1", count: 0 };
        let store = createStore(initState);
        let substore = store.map('name');
        substore.action().subscribe((s, a) => { return s; });
        let state = null;
        let substate = null;
        store.observable().subscribe(s => state = s);
        substore.observable().subscribe(s => substate = s);
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch({ name: "test" });
        chai_1.expect(substate).to.be.eq('1', "name should be the same");
        chai_1.expect(state).to.be.eq(initState, "the state should still be the same");
        chai_1.expect(state.name).to.be.eq("1", "name should notbe change by dispatch action");
    });
    it('store change any action value subStore by mapFunction function', () => {
        let initState = { name: "1", count: 0 };
        let store = createStore(initState);
        let substore = store.mapFunction('name', (s) => s.name, (s, p) => { p.name = s; return p; });
        substore.action().subscribe((s, a) => { return a.name; });
        let state = null;
        let substate = null;
        store.observable().subscribe(s => state = s);
        substore.observable().subscribe(s => substate = s);
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch({ name: "test" });
        chai_1.expect(substate).to.be.eq('test', "name in substate should be change by dispatch action");
        chai_1.expect(state).to.be.not.eq(initState, "the state should be a new one s");
        chai_1.expect(state.name).to.be.eq("test", "name should be change by dispatch action");
    });
    it('store change any action value subStore deep', () => {
        let initState = { parent: { name: "1" } };
        let store = createStore(initState);
        let substore = store.map('parent.name');
        substore.action().subscribe((s, a) => { return a.name; });
        let state = null;
        let substate = null;
        store.observable().subscribe(s => state = s);
        substore.observable().subscribe(s => substate = s);
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch({ name: "test" });
        chai_1.expect(substate).to.be.eq('test', "name in substate should be change by dispatch action");
        chai_1.expect(state).to.be.not.eq(initState, "the state should be a new one s");
        chai_1.expect(state.parent.name).to.be.eq("test", "name should be change by dispatch action");
    });
    it('store change any action value subStore of substore', () => {
        let initState = { parent: { name: "1" } };
        let store = createStore(initState);
        let substore = store.map('parent').map('name');
        substore.action().subscribe((s, a) => { return a.name; });
        let state = null;
        let substate = null;
        store.observable().subscribe(s => state = s);
        substore.observable().subscribe(s => substate = s);
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch({ name: "test" });
        chai_1.expect(substate).to.be.eq('test', "name in substate should be change by dispatch action");
        chai_1.expect(state).to.be.not.eq(initState, "the state should be a new one s");
        chai_1.expect(state.parent.name).to.be.eq("test", "name should be change by dispatch action");
    });
    it('store change any action value subStore of substore by mapFunction function', () => {
        let initState = { parent: { name: "1" } };
        let store = createStore(initState);
        let substore = store.mapFunction('parent', (s) => s.parent, (s, p) => { p.parent = s; return p; })
            .mapFunction('name', (s) => s.name, (s, p) => { p.name = s; return p; });
        substore.action().subscribe((s, a) => { return a.name; });
        let state = null;
        let substate = null;
        store.observable().subscribe(s => state = s);
        substore.observable().subscribe(s => substate = s);
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch({ name: "test" });
        chai_1.expect(substate).to.be.eq('test', "name in substate should be change by dispatch action");
        chai_1.expect(state).to.be.not.eq(initState, "the state should be a new one s");
        chai_1.expect(state.parent.name).to.be.eq("test", "name should be change by dispatch action");
    });
    it('store change any action value subStore using reducer', () => {
        let initState = { name: "1", count: 0 };
        let store = createStore(initState);
        let substore = store.map('name');
        substore.actionByType(EventName).subscribeReducer(new NameReducer());
        let state = null;
        let substate = null;
        store.observable().subscribe(s => state = s);
        substore.observable().subscribe(s => substate = s);
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch(new EventName("test"));
        chai_1.expect(substate).to.be.eq('test', "name in substate should be change by dispatch action");
        chai_1.expect(state).to.be.not.eq(initState, "the state should be a new one s");
        chai_1.expect(state.name).to.be.eq("test", "name should be change by dispatch action");
    });
    it('store change filtered action value subStore using function', () => {
        let initState = { name: "1", count: 0 };
        let store = createStore(initState);
        let substore = store.map('name');
        substore.actionByType(EventName)
            .filter(e => e.name !== 'ignore')
            .filter(e => e.name !== null)
            .subscribe((s, e) => e.name);
        let state = null;
        let substate = null;
        store.observable().subscribe(s => state = s);
        substore.observable().subscribe(s => substate = s);
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch(new EventName(null));
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch(new EventName("ignore"));
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch(new EventName("test"));
        chai_1.expect(substate).to.be.eq('test', "name in substate should be change by dispatch action");
        chai_1.expect(state).to.be.not.eq(initState, "the state should be a new one s");
        chai_1.expect(state.name).to.be.eq("test", "name should be change by dispatch action");
    });
    it('store change filtered action value subStore using reducer', () => {
        let initState = { name: "1", count: 0 };
        let store = createStore(initState);
        let substore = store.map('name');
        substore.actionByType(EventName)
            .filter(e => e.name !== 'ignore')
            .filter(e => e.name !== null)
            .subscribeReducer(new NameReducer());
        let state = null;
        let substate = null;
        store.observable().subscribe(s => state = s);
        substore.observable().subscribe(s => substate = s);
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch(new EventName(null));
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch(new EventName("ignore"));
        chai_1.expect(state).to.be.eq(initState);
        chai_1.expect(substate).to.be.eq('1');
        substore.dispatch(new EventName("test"));
        chai_1.expect(substate).to.be.eq('test', "name in substate should be change by dispatch action");
        chai_1.expect(state).to.be.not.eq(initState, "the state should be a new one s");
        chai_1.expect(state.name).to.be.eq("test", "name should be change by dispatch action");
    });
    it('change in a substore should not trigger distinct substore', () => {
        let initState = { name: 0, count: 0 };
        let store = createStore(initState);
        let countStoreUpdate = 0;
        let storeUpdate = 0;
        let nameUpdate = 0;
        store.observable().subscribe(s => storeUpdate += 1);
        store.map("count").observable().subscribe(s => countStoreUpdate += 1);
        store.map("name").observable().subscribe(s => nameUpdate += 1);
        store.map("count").action().subscribe((s, a) => s + 1);
        store.map("name").action().subscribe((s, a) => s + 1);
        store.map("name").dispatch({});
        chai_1.expect(countStoreUpdate).to.be.eq(1);
        chai_1.expect(storeUpdate).to.be.eq(2);
        chai_1.expect(nameUpdate).to.be.eq(2);
        store.map("count").dispatch({});
        chai_1.expect(countStoreUpdate).to.be.eq(2);
        chai_1.expect(storeUpdate).to.be.eq(3);
        chai_1.expect(nameUpdate).to.be.eq(2);
        store.map("count").dispatch({});
        chai_1.expect(countStoreUpdate).to.be.eq(3);
        chai_1.expect(storeUpdate).to.be.eq(4);
        chai_1.expect(nameUpdate).to.be.eq(2);
    });
});
//# sourceMappingURL=Store.spec.js.map